% Generation of graphics for delay time distribution
%
% Signal path is:
% GPIO0 -> Inputframe (Inp) -> Controller -> Outputframe (Out) -> GPIO1
% Only raising signals are considered
% Data is read from the subdirectory "Data" form a .csv file
% Graphics are stored in subdirectory "Figures" as a .jpg file
% Report is generated into the subdirectory "log" as a simple .html file
% All files have the same "filename"
% version 12.2.2019, Max Felser
clear all;
close all;

% installed package required:
%pkg install -forge io
pkg load io

% Data is available as a .csv file with 
filename = 'Profinet_test';

% Position of Information in the table, adapt if required
Inp_Bit_Column = 3;  % cell with string "0" or "1" for Input
Out_Bit_Column = 5;  % cell with string "0" or "1" for Output
GPIO0_Bit_Column = 7; % cell with string "rising edge." for GPIO0
GPIO1_Bit_Column = 9; % cell with string "rising edge." for GPIO1

% start log into a new html file
log_file = fopen(strcat("Log/",filename,".html"),"wt");
fprintf(log_file,"<html><body>\n<h1>Measurement report: %s</h1>\n",filename);
printf("Start log - open data file ...\n");

% Load CSV-file(converted to Cells) change delimiter to ';', skip one headline
T = csv2cell(strcat('Data/', filename,'.csv'),1,';');
fprintf(log_file,"<p><a href=..\\\Data\\%s.csv>Data Source</a></p>\n",filename);
printf("data loaded - start processing ");

% Put Time into seperate Vector
Time_String = T(1:end,2);
fprintf(log_file,"<p>%d events loaded</p>",length(Time_String));
printf("%d events ...\n",length(Time_String));

% Extract Index of Rising Edge Inputframe (Device-to-Controller) 
% Mark empty cells
Cell_empty = cellfun("isempty",T(1:end,Inp_Bit_Column));
% Collect Indexes of non empty cells
Index1 = 1;
for Index = 1 : rows(Cell_empty)
  if !(Cell_empty(Index))
    Input_Status(Index1) = T(Index,Inp_Bit_Column);
    Input_Index(Index1++) = Index; 
  endif
end
% Find rising edge of Input
Index1 = 1;
for Index = 2 : columns(Input_Status)
  if (cell2mat(Input_Status(Index)) > cell2mat(Input_Status(Index-1)))
    Input_Rising(Index1++) = Input_Index(Index); 
  endif
end

% Extract Index of Rising Edge Outputframe (Controller-to-Device)
% Mark empty cells
Cell_empty = cellfun("isempty",T(1:end,Out_Bit_Column));
% Collect indexes of non empty cells
Index1 = 1;
for Index = 1 : rows(Cell_empty)
  if !(Cell_empty(Index))
    Output_Status(Index1) = T(Index,Out_Bit_Column);
    Output_Index(Index1++) = Index; 
  endif
end
% Find rising edge of Lampe
Index1 = 1;
for Index = 2 : columns(Output_Status)
  if (cell2mat(Output_Status(Index)) > cell2mat(Output_Status(Index-1)))
    Output_Rising(Index1++) = Output_Index(Index); 
  endif
end

% Extract Index of Rising Edge GPIO 0(Connected to Input of IO-Module)
% Mark empty cells
Cell_empty = cellfun("isempty",T(1:end,GPIO0_Bit_Column));
% Collect indexes of non empty cells
Index1 = 1;
for Index = 1 : rows(Cell_empty)
  if !(Cell_empty(Index))
    GPIO0_Rising(Index1++) = Index; 
  endif
end

% Extract Index of Rising Edge GPIO 1(Connected to Output of IO-Module)
% Mark empty cells
Cell_empty = cellfun("isempty",T(1:end,GPIO1_Bit_Column));
% Collect indexes of non empty cells
Index1 = 1;
for Index = 1 : rows(Cell_empty)
  if !(Cell_empty(Index))
    GPIO1_Rising(Index1++) = Index; 
  endif
end

% Calculate Delay Times in ms
Total_Delay = FindDelay(GPIO0_Rising, GPIO1_Rising, Time_String)*1000;
IO_Read_Delay = FindDelay(GPIO0_Rising, Input_Rising, Time_String)*1000;
Raspy_Calc_Delay = FindDelay(Input_Rising, Output_Rising, Time_String)*1000;
IO_Write_Delay = FindDelay(Output_Rising, GPIO1_Rising, Time_String)*1000;

printf("Values calculated - draw graphics ... \n");

% Create Diagramms
figure("Name",filename);
fprintf(log_file,'<table border="1"><tr><th>Measurement</th><th>Num</th><th>Mean</th><th>Min</th><th>Max</th></tr>\n');

% Plot Total Delay
subplot(2, 2, 1);
hist(Total_Delay);
xlabel("Time /ms");
ylabel("Num of Samples /1");
title("Total Delay (GPIO0 -> GPIO1)");
line([mean(Total_Delay), mean(Total_Delay)], ylim, 'LineWidth', 1, 'Color', 'r');
% Report to Command Window
%fprintf(log_file,"Total Delay (GPIO0 -> GPIO1):\nN = %d  mean = %.2fms  min = %.2fms  max = %.2fms\n\n", 
fprintf(log_file,"<tr><td>Total Delay (GPIO0 -> GPIO1)</td><td> %d </td><td> %.2fms </td><td> %.2fms </td><td> %.2fms</td></tr>\n", 
    length(Total_Delay), mean(Total_Delay), 
    min(Total_Delay), max(Total_Delay));

% Plot Input Delay
subplot(2, 2, 2);
hist(IO_Read_Delay);
xlabel("Time /ms");
ylabel("Num of Samples /1");
title("Input Delay (GPIO0 -> Inputframe)");
line([mean(IO_Read_Delay), mean(IO_Read_Delay)], ylim, 'LineWidth', 1, 'Color', 'r');
% report to command window
%fprintf(log_file,"Input Delay (GPIO0 -> Inputframe):\nN = %d  mean = %.2fms  min = %.2fms  max = %.2fms\n\n", 
fprintf(log_file,"<tr><td>Input Delay (GPIO0 -> Inputframe)</td><td> %d </td><td> %.2fms </td><td> %.2fms </td><td> %.2fms</td></tr>\n", 
     length(IO_Read_Delay), mean(IO_Read_Delay), 
     min(IO_Read_Delay), max(IO_Read_Delay));

% Plot Controller Delay (Inputframe -> Ouputframe)
subplot(2, 2, 3);
hist(Raspy_Calc_Delay);
xlabel("Time /ms");
ylabel("Num of Samples /1");
title("Controller Delay (Inputframe -> Ouputframe)");
line([mean(Raspy_Calc_Delay), mean(Raspy_Calc_Delay)], ylim, 'LineWidth', 1, 'Color', 'r');
%fprintf(log_file,"Controller Delay (Inputframe -> Ouputframe):\nN = %d  mean = %.2fms  min = %.2fms  max = %.2fms\n\n", 
fprintf(log_file,"<tr><td>Controller Delay (Inputframe -> Ouputframe)</td><td> %d </td><td> %.2fms </td><td> %.2fms </td><td> %.2fms</td></tr>\n", 
     length(Raspy_Calc_Delay), mean(Raspy_Calc_Delay), 
     min(Raspy_Calc_Delay),max(Raspy_Calc_Delay));

%Plot Ouput Delay (Outputframe -> GPIO1)
subplot(2, 2, 4);
hist(IO_Write_Delay);
xlabel("Time /ms");
ylabel("Num of Samples /1");
title("Output Delay (Outputframe -> GPIO1)");
line([mean(IO_Write_Delay), mean(IO_Write_Delay)], ylim, 'LineWidth', 1, 'Color', 'r');
%report to command window
%fprintf(log_file,"Output Delay (Outputframe -> GPIO1):\nN = %d  mean = %.2fms  min = %.2fms  max = %.2fms\n", 
fprintf(log_file,"<tr><td>Output Delay (Outputframe -> GPIO1)</td><td> %d </td><td> %.2fms </td><td> %.2fms </td><td> %.2fms</td></tr>\n", 
     length(IO_Write_Delay), mean(IO_Write_Delay), 
     min(IO_Write_Delay), max(IO_Write_Delay));

% Save Diagrams to .jpg file
%print(strcat(filename,'.jpg'),djpg);
h = gcf;
saveas(h,strcat("Figures/",filename,".jpg"));

printf("graphic stored - close files:");

fprintf(log_file,"</table>\n<img src=..\\\Figures\\%s.jpg>\n</body></html>",filename);
fclose(log_file);
printf("OK\n\n");